// tinyasm.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "tinyasm.h"
#include "assembler.h"

#pragma warning(disable: 4996)

char* sourceFilename = NULL;
char* outputFilename = NULL;

int main(int argc, char* argv[])
{
	if (!ParseCommandLine(argc, argv))
	{
		return -1;
	}

	FILE *pSource = fopen(sourceFilename, "r");
	if (!pSource)
	{
		printf("Could not open source file %s\n", sourceFilename);
		return -1;
	}

	FILE *pOutput = fopen(outputFilename, "w");
	if (!pOutput)
	{
		printf("Could not open output file %s\n", outputFilename);
		return -1;
	}

	Assembler assembler;
	uint8vec outputBuffer;
	if (assembler.Assemble(sourceFilename, pSource, &outputBuffer))
	{
		// write the output file
		int outputSize = outputBuffer.size();
		for (int i=0; i<outputSize; i++)
		{
			OutputHex(pOutput, outputBuffer[i]);
		}

		printf("Wrote %s\n", outputFilename);
	}

    fclose(pSource);
	fclose(pOutput);

	return 0;
}

void OutputHex(FILE* pFile, int val)
{
	int nibble = (val & 0xFF) >> 4;
	fputc(nibble <= 9 ? '0' + nibble : 'A' + nibble - 10, pFile);

	nibble = val & 0xF;
	fputc(nibble <= 9 ? '0' + nibble : 'A' + nibble - 10, pFile);

	fputc(' ', pFile);
}

bool ParseCommandLine(int argc, char* argv[])
{
	for (int i=1; i < argc; i++)
	{
		char* arg = argv[i];

		if (arg[0] == '-')
		{
			switch(arg[1])
			{
			case 'o':
				if (i < argc-1)
				{
					outputFilename = argv[i+1];
					i++;
				}
				break;

			default:
				printf("Unrecognized option %s\n", arg);
				Usage(argv[0]);
				return false;
			}
		}
		else
		{
			if (sourceFilename == NULL)
			{
				sourceFilename = arg;
			}
			else
			{
				printf("Unrecognized option %s\n", arg);
				Usage(argv[0]);
				return false;
			}
		}
	}

	if (sourceFilename == NULL)
	{
		printf("No source file specified.\n");
		Usage(argv[0]);
		return false;
	}

	if (outputFilename == NULL)
	{
		printf("No output file specified.\n");
		Usage(argv[0]);
		return false;
	}

	return true;
}

void Usage(char* name)
{
	printf("%s -o outfile infile\n", name);
}
